/**
 * TNTConcept Easy Enterprise Management by Autentia Real Bussiness Solution S.L.
 * Copyright (C) 2007 Autentia Real Bussiness Solution S.L.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.autentia.tnt.manager.account;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.autentia.tnt.businessobject.AccountEntryGroup;
import com.autentia.tnt.businessobject.AccountEntryType;
import com.autentia.tnt.dao.SortCriteria;
import com.autentia.tnt.dao.hibernate.AccountEntryGroupDAO;
import com.autentia.tnt.dao.hibernate.AccountEntryTypeDAO;
import com.autentia.tnt.dao.search.AccountEntryTypeSearch;
import com.autentia.tnt.util.SpringUtils;



public class AccountEntryTypeManager {


/* AccountEntryType - generated by stajanov (do not edit/delete) */



  /** Logger */
  private static final Log log = LogFactory.getLog(AccountEntryTypeManager.class);

  /** AccountEntryType DAO **/
  private AccountEntryTypeDAO accountEntryTypeDAO;

  /**
   * Get default AccountEntryTypeManager as defined in Spring's configuration file.
   * @return the default singleton AccountEntryTypeManager
   */
  public static AccountEntryTypeManager getDefault()
  {
    return (AccountEntryTypeManager)SpringUtils.getSpringBean("managerAccountEntryType");
  }

  /** 
   * Empty constructor needed by CGLIB (Spring AOP)
   */
  protected AccountEntryTypeManager()
  {
  }
	
  /** 
   * Default constructor 
   * @deprecated do not construct managers alone: use Spring's declared beans
   */
  public AccountEntryTypeManager( AccountEntryTypeDAO accountEntryTypeDAO )
  {
    this.accountEntryTypeDAO = accountEntryTypeDAO;
  }

  /**
   * List accountEntryTypes. 
   * @param search search filter to apply
   * @param sort sorting criteria
   * @return the list of all accountEntryTypes sorted by requested criterion
   */
  public List<AccountEntryType> getAllEntities(AccountEntryTypeSearch search, SortCriteria sort){
    return accountEntryTypeDAO.search( search, sort );
  }
  
  /**
   * Get accountEntryType by primary key.
   * @return accountEntryType selected by id.
   */
  public AccountEntryType getEntityById(int id){
    return accountEntryTypeDAO.getById(id);	    
  }
	
  /**
   * Insert accountEntryType. 
   */
  public void insertEntity(AccountEntryType accountEntryType) {
    accountEntryTypeDAO.insert(accountEntryType);
  }
	 
  /**
   * Update accountEntryType. 
   */
  public void updateEntity(AccountEntryType accountEntryType) {
    accountEntryTypeDAO.update(accountEntryType);
  }

  /**
   * Delete accountEntryType. 
   */
  public void deleteEntity(AccountEntryType accountEntryType) {
    accountEntryTypeDAO.delete(accountEntryType);
  }

/* AccountEntryType - generated by stajanov (do not edit/delete) */
}
