/**
 * TNTConcept Easy Enterprise Management by Autentia Real Bussiness Solution S.L.
 * Copyright (C) 2007 Autentia Real Bussiness Solution S.L.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.autentia.tnt.manager.admin;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.autentia.tnt.businessobject.ContractType;
import com.autentia.tnt.dao.SortCriteria;
import com.autentia.tnt.dao.hibernate.ContractTypeDAO;
import com.autentia.tnt.dao.search.ContractTypeSearch;
import com.autentia.tnt.util.SpringUtils;



public class ContractTypeManager {



/* Department - generated by stajanov (do not edit/delete) */



  /** Logger */
  private static final Log log = LogFactory.getLog(ContractTypeManager.class);

  /** ContractType DAO **/
  private ContractTypeDAO contractTypeDAO;

  /**
   * Get default ContractTypeManager as defined in Spring's configuration file.
   * @return the default singleton ContractTypeManager
   */
  public static ContractTypeManager getDefault()
  {
    return (ContractTypeManager)SpringUtils.getSpringBean("managerContractType");
  }

  /** 
   * Empty constructor needed by CGLIB (Spring AOP)
   */
  protected ContractTypeManager()
  {
  }
	
  /** 
   * Default constructor 
   * @deprecated do not construct managers alone: use Spring's declared beans
   */
  public ContractTypeManager( ContractTypeDAO contractTypeDAO )
  {
    this.contractTypeDAO = contractTypeDAO;
  }

  /**
   * List contractTypes. 
   * @param search search filter to apply
   * @param sort sorting criteria
   * @return the list of all contractTypes sorted by requested criterion
   */
  public List<ContractType> getAllEntities(ContractTypeSearch search, SortCriteria sort){
    return contractTypeDAO.search( search, sort );
  }
  
  /**
   * Get contractType by primary key.
   * @return contractType selected by id.
   */
  public ContractType getEntityById(int id){
    return contractTypeDAO.getById(id);	    
  }
	
  /**
   * Insert contractType. 
   */
  public void insertEntity(ContractType contractType) {
    contractTypeDAO.insert(contractType);
  }
	 
  /**
   * Update contractType. 
   */
  public void updateEntity(ContractType contractType) {
    contractTypeDAO.update(contractType);
  }

  /**
   * Delete contractType. 
   */
  public void deleteEntity(ContractType contractType) {
    contractTypeDAO.delete(contractType);
  }

/* Department - generated by stajanov (do not edit/delete) */

	
}
