/**
 * TNTConcept Easy Enterprise Management by Autentia Real Bussiness Solution S.L.
 * Copyright (C) 2007 Autentia Real Bussiness Solution S.L.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.autentia.tnt.manager.commissioning;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.autentia.tnt.businessobject.CommissioningFile;
import com.autentia.tnt.dao.SortCriteria;
import com.autentia.tnt.dao.hibernate.CommissioningFileDAO;
import com.autentia.tnt.dao.search.CommissioningFileSearch;
import com.autentia.tnt.util.SpringUtils;

/* CreditTitle - generated by stajanov (do not edit/delete) */


public class CommissioningFileManager {
  /** Logger */
  private static final Log log = LogFactory.getLog(CommissioningFileManager.class);

  /** CommissioningFile DAO **/
  private CommissioningFileDAO commissioningFileDAO;

  /**
   * Get default CommissioningFileManager as defined in Spring's configuration file.
   * @return the default singleton CommissioningFileManager
   */
  public static CommissioningFileManager getDefault()
  {
    return (CommissioningFileManager)SpringUtils.getSpringBean("managerCommissioningFile");
  }

  /** 
   * Empty constructor needed by CGLIB (Spring AOP)
   */
  protected CommissioningFileManager()
  {
  }
	
  /** 
   * Default constructor 
   * @deprecated do not construct managers alone: use Spring's declared beans
   */
  public CommissioningFileManager( CommissioningFileDAO commissioningFileDAO )
  {
    this.commissioningFileDAO = commissioningFileDAO;
  }

  /**
   * List commissioningFiles. 
   * @param search search filter to apply
   * @param sort sorting criteria
   * @return the list of all commissioningFiles sorted by requested criterion
   */
  public List<CommissioningFile> getAllEntities(CommissioningFileSearch search, SortCriteria sort){
    return commissioningFileDAO.search( search, sort );
  }
  
  /**
   * Get commissioningFile by primary key.
   * @return commissioningFile selected by id.
   */
  public CommissioningFile getEntityById(int id){
    return commissioningFileDAO.getById(id);	    
  }
	
  /**
   * Insert commissioningFile. 
   */
  public void insertEntity(CommissioningFile commissioningFile) {
    commissioningFileDAO.insert(commissioningFile);
  }
	 
  /**
   * Update commissioningFile. 
   */
  public void updateEntity(CommissioningFile commissioningFile) {
    commissioningFileDAO.update(commissioningFile);
  }

  /**
   * Delete commissioningFile. 
   */
  public void deleteEntity(CommissioningFile commissioningFile) {
    commissioningFileDAO.delete(commissioningFile);
  }

/* CreditTitle - generated by stajanov (do not edit/delete) */
}