/**
 * TNTConcept Easy Enterprise Management by Autentia Real Bussiness Solution S.L.
 * Copyright (C) 2007 Autentia Real Bussiness Solution S.L.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.autentia.tnt.bean.publish;

import java.util.*;
import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlDataTable;
import javax.faces.model.*;

import org.apache.commons.logging.*;

import com.autentia.tnt.bean.*;
import com.autentia.tnt.businessobject.*;
import com.autentia.tnt.dao.*;
import com.autentia.tnt.dao.search.*;
import com.autentia.tnt.manager.admin.UserManager;
import com.autentia.tnt.manager.publish.TutorialManager;
import com.autentia.tnt.manager.security.Permission;
import com.autentia.tnt.upload.*;
import com.autentia.tnt.util.*;

import org.acegisecurity.acls.domain.BasePermission;

/**
 * UI bean for Tutorial objects.
 * @author stajanov code generator
 */
public class TutorialBean extends BaseBean 
{
    /** Serial version field */
    private static final long serialVersionUID = -1L;
    
/* tutorial - generated by stajanov (do not edit/delete) */


















  /** Logger */
  private static final Log log = LogFactory.getLog(TutorialBean.class);

 /** Active search object */
  private TutorialSearch search = new TutorialSearch();

  /** Manager */
  private static TutorialManager manager = TutorialManager.getDefault();
  
  /** Upload service */
  private static final Uploader uploader = UploaderFactory.getInstance("tutorial");
    
                                                          
  /** Active Tutorial object */
  private Tutorial tutorial;
  
  /** Default sort column */
  private String sortColumn = "name";
    
  /** Default sort order */
  private boolean sortAscending = true;

      /** Quick search letter for ABC pager control */
    private Character letter;
  
  /**
   * List tutorials. Order depends on Faces parameter sort.
   * @return the list of all tutorials sorted by requested criterion
   */
  public List<Tutorial> getAll(){
    return manager.getAllEntities(search, new SortCriteria(sortColumn, sortAscending) );      
  }

  // Getters to list possible values of related entities
      
    
  /**
   * Get the list of all users
   * @return the list of all users
   */
  public List<SelectItem> getUsers(){  
    List<User> refs = UserManager.getDefault().getAllEntities( null, new SortCriteria("name") );
	  
    ArrayList<SelectItem> ret = new ArrayList<SelectItem>();  
    for( User ref : refs ){
      ret.add( new SelectItem( ref, ref.getName() ) );
    }
    return ret;	
  }
        
  // Getters to list possible values of enum fields
                                            

  // Methods to create/remove instances of one-to-many entities (slave entities)
  
  /**
   * Whether or not create button is available for user
   * @return true if user can create objects of type Tutorial
   */
  public boolean isCreateAvailable()
  {
    return SpringUtils.isRolePermissionGranted(Permission.Entity_Create(Tutorial.class));
  }

  /**
   * Whether or not edit button is available for user
   * @return true if user can edit current object
   */
  public boolean isEditAvailable()
  {
    return SpringUtils.isAclPermissionGranted(tutorial,BasePermission.WRITE);
  }

  /**
   * Whether or not delete button is available for user
   * @return true if user can delete current object
   */
  public boolean isDeleteAvailable()
  {
    return (tutorial.getId()!=null) &&
	   SpringUtils.isAclPermissionGranted(tutorial,BasePermission.DELETE);
  }

  /**
   * Go to create page
   * @return forward to CREATE page
   */
  public String create(){
    tutorial = new Tutorial();
       
    return NavigationResults.CREATE;
  }

  /**
   * Go to detail page
   * @return forward to DETAIL page
   */
  public String detail(){
    Integer id = Integer.parseInt( FacesUtils.getRequestParameter(ROW_ID) );
    tutorial = manager.getEntityById(id);

    return SpringUtils.isAclPermissionGranted( tutorial, BasePermission.WRITE )
	    ? NavigationResults.EDIT
	    : NavigationResults.DETAIL;
  }

  /**
   * Save bean and stay on it
   * @return forward to list page
   */
  public String save(){
  
      doBeforeSave();
  
      if( tutorial.getId()==null ){
        manager.insertEntity(tutorial);
      } else {
        manager.updateEntity(tutorial);
      }
      

                                                                                                                      
      // Calls an after save action
      String result = doAfterSave(NavigationResults.LIST);

      // Unselect object
      tutorial = null;

      return result;
  }
    
  /**
   * Delete bean and go back to beans list
   * @return forward to LIST page
   */
  public String delete(){
    manager.deleteEntity(tutorial);
    tutorial = null;
    return NavigationResults.LIST;
  }

  /**
   * Go back to beans list
   * @return forward to LIST page
   */
  public String list(){
    return NavigationResults.LIST;
  }

  /**
   * Reset search criteria
   * @return forward to LIST page
   */
  public String reset(){
  	search.reset();
	return list();    
  }

  /**
   * Go to search page
   * @return forward to SEARCH page
   */
  public String search(){
    return NavigationResults.SEARCH;
  }

  /** 
   * Check if we have an active object.
   * @return true is an object is selected
   */
  public boolean isTutorialSelected(){
    return tutorial!=null;
  }

  // Getters and setters to manipulate sorting
  public boolean isSortAscending() {
    return sortAscending;
  }
  public void setSortAscending(boolean sortAscending) {
    this.sortAscending = sortAscending;
  }
  public String getSortColumn() {
    return sortColumn;
  }
  public void setSortColumn(String sortColumn) {
    this.sortColumn = sortColumn;
  }

  // Getters and setters to handle search
  public TutorialSearch getSearch(){
    return search;
  }
      
    
    
    
    
    public String getSearchName(){
        return search.getName();
    }
    public void setSearchName( String val ){
        if( search.isNameSet() ) {
          search.setName( val );
        }
    }
    public boolean isSearchNameValid(){
        return search.isNameSet();
    }
    public void setSearchNameValid( boolean val ){
        if( val ){
          search.setName( search.getName() );
        } else {
          search.unsetName();
        }
    }
        
    
    
    
    
    public String getSearchDescription(){
        return search.getDescription();
    }
    public void setSearchDescription( String val ){
        if( search.isDescriptionSet() ) {
          search.setDescription( val );
        }
    }
    public boolean isSearchDescriptionValid(){
        return search.isDescriptionSet();
    }
    public void setSearchDescriptionValid( boolean val ){
        if( val ){
          search.setDescription( search.getDescription() );
        } else {
          search.unsetDescription();
        }
    }
        
    
        
      
    public Date getSearchStartMaxDeliveryDate(){
        return search.getStartMaxDeliveryDate();
    }
    public void setSearchStartMaxDeliveryDate( Date val ){
        if( val!=null ){
          search.setStartMaxDeliveryDate( val );
        } else {
          search.unsetStartMaxDeliveryDate();
        }
    }
    public boolean isSearchStartMaxDeliveryDateValid(){
        return search.isStartMaxDeliveryDateSet();
    }
    public void setSearchStartMaxDeliveryDateValid( boolean val ){
        if( val ){
          search.setStartMaxDeliveryDate( search.getStartMaxDeliveryDate() );
        } else {
          search.unsetStartMaxDeliveryDate();
        }
    }
    public Date getSearchEndMaxDeliveryDate(){
        return search.getEndMaxDeliveryDate();
    }
    public void setSearchEndMaxDeliveryDate( Date val ){
        if( val!=null ){
          search.setEndMaxDeliveryDate( val );
        } else {
          search.unsetEndMaxDeliveryDate();
        }
    }
    public boolean isSearchEndMaxDeliveryDateValid(){
        return search.isEndMaxDeliveryDateSet();
    }
    public void setSearchEndMaxDeliveryDateValid( boolean val ){
        if( val ){
          search.setEndMaxDeliveryDate( search.getEndMaxDeliveryDate() );
        } else {
          search.unsetEndMaxDeliveryDate();
        }
    }

        
    
        
      
    public Date getSearchStartEndDate(){
        return search.getStartEndDate();
    }
    public void setSearchStartEndDate( Date val ){
        if( val!=null ){
          search.setStartEndDate( val );
        } else {
          search.unsetStartEndDate();
        }
    }
    public boolean isSearchStartEndDateValid(){
        return search.isStartEndDateSet();
    }
    public void setSearchStartEndDateValid( boolean val ){
        if( val ){
          search.setStartEndDate( search.getStartEndDate() );
        } else {
          search.unsetStartEndDate();
        }
    }
    public Date getSearchEndEndDate(){
        return search.getEndEndDate();
    }
    public void setSearchEndEndDate( Date val ){
        if( val!=null ){
          search.setEndEndDate( val );
        } else {
          search.unsetEndEndDate();
        }
    }
    public boolean isSearchEndEndDateValid(){
        return search.isEndEndDateSet();
    }
    public void setSearchEndEndDateValid( boolean val ){
        if( val ){
          search.setEndEndDate( search.getEndEndDate() );
        } else {
          search.unsetEndEndDate();
        }
    }

        
    
    
    
    
    public Integer getSearchDepartmentId(){
        return search.getDepartmentId();
    }
    public void setSearchDepartmentId( Integer val ){
        if( search.isDepartmentIdSet() ) {
          search.setDepartmentId( val );
        }
    }
    public boolean isSearchDepartmentIdValid(){
        return search.isDepartmentIdSet();
    }
    public void setSearchDepartmentIdValid( boolean val ){
        if( val ){
          search.setDepartmentId( search.getDepartmentId() );
        } else {
          search.unsetDepartmentId();
        }
    }
        
    
        
      
    public Date getSearchStartInsertDate(){
        return search.getStartInsertDate();
    }
    public void setSearchStartInsertDate( Date val ){
        if( val!=null ){
          search.setStartInsertDate( val );
        } else {
          search.unsetStartInsertDate();
        }
    }
    public boolean isSearchStartInsertDateValid(){
        return search.isStartInsertDateSet();
    }
    public void setSearchStartInsertDateValid( boolean val ){
        if( val ){
          search.setStartInsertDate( search.getStartInsertDate() );
        } else {
          search.unsetStartInsertDate();
        }
    }
    public Date getSearchEndInsertDate(){
        return search.getEndInsertDate();
    }
    public void setSearchEndInsertDate( Date val ){
        if( val!=null ){
          search.setEndInsertDate( val );
        } else {
          search.unsetEndInsertDate();
        }
    }
    public boolean isSearchEndInsertDateValid(){
        return search.isEndInsertDateSet();
    }
    public void setSearchEndInsertDateValid( boolean val ){
        if( val ){
          search.setEndInsertDate( search.getEndInsertDate() );
        } else {
          search.unsetEndInsertDate();
        }
    }

        
    
        
      
    public Date getSearchStartUpdateDate(){
        return search.getStartUpdateDate();
    }
    public void setSearchStartUpdateDate( Date val ){
        if( val!=null ){
          search.setStartUpdateDate( val );
        } else {
          search.unsetStartUpdateDate();
        }
    }
    public boolean isSearchStartUpdateDateValid(){
        return search.isStartUpdateDateSet();
    }
    public void setSearchStartUpdateDateValid( boolean val ){
        if( val ){
          search.setStartUpdateDate( search.getStartUpdateDate() );
        } else {
          search.unsetStartUpdateDate();
        }
    }
    public Date getSearchEndUpdateDate(){
        return search.getEndUpdateDate();
    }
    public void setSearchEndUpdateDate( Date val ){
        if( val!=null ){
          search.setEndUpdateDate( val );
        } else {
          search.unsetEndUpdateDate();
        }
    }
    public boolean isSearchEndUpdateDateValid(){
        return search.isEndUpdateDateSet();
    }
    public void setSearchEndUpdateDateValid( boolean val ){
        if( val ){
          search.setEndUpdateDate( search.getEndUpdateDate() );
        } else {
          search.unsetEndUpdateDate();
        }
    }

          
    
    
        
    
    public User getSearchUser(){
        return search.getUser();
    }
    public void setSearchUser( User val ){
        if( search.isUserSet() ) {
          search.setUser( val );
        }
    }
    public boolean isSearchUserValid(){
        return search.isUserSet();
    }
    public void setSearchUserValid( boolean val ){
        if( val ){
          search.setUser( search.getUser() );
        } else {
          search.unsetUser();
        }
    }
        
      /** Handle an ABC pager letter click: filter objects by specified starting letter */
    public void letterClicked()
    {
      if( letter!=null ){
      	UIComponent comp = FacesUtils.getComponent("tutorials:list");
        HtmlDataTable tabla = (HtmlDataTable) comp;
        tabla.setFirst(0);
      	
        search.setName( letter+"%" );
      } else {
        search.unsetName();
      }
    }

    public Character getLetter()
    {
      return letter;
    }

    public void setLetter( Character letter )
    {
      this.letter = letter;
    }
  
  // Getters and setters to handle uploads
                                                          
  // Getters and setters to manipulate active Tutorial object
  public java.lang.Integer getId() {
      return tutorial.getId();
  }
      
    
    
    public String getName() {
    return tutorial.getName();
  }
  public void setName( String name ) {
    tutorial.setName( name );
  }
        
    
    
    public String getDescription() {
    return tutorial.getDescription();
  }
  public void setDescription( String description ) {
    tutorial.setDescription( description );
  }
        
    
    
    public Date getMaxDeliveryDate() {
    return tutorial.getMaxDeliveryDate();
  }
  public void setMaxDeliveryDate( Date maxDeliveryDate ) {
    tutorial.setMaxDeliveryDate( maxDeliveryDate );
  }
        
    
    
    public Date getEndDate() {
    return tutorial.getEndDate();
  }
  public void setEndDate( Date endDate ) {
    tutorial.setEndDate( endDate );
  }
        
    
    
    public Integer getDepartmentId() {
    return tutorial.getDepartmentId();
  }
  public void setDepartmentId( Integer departmentId ) {
    tutorial.setDepartmentId( departmentId );
  }
        
    
    
    public Date getInsertDate() {
    return tutorial.getInsertDate();
  }
  public void setInsertDate( Date insertDate ) {
    tutorial.setInsertDate( insertDate );
  }
        
    
    
    public Date getUpdateDate() {
    return tutorial.getUpdateDate();
  }
  public void setUpdateDate( Date updateDate ) {
    tutorial.setUpdateDate( updateDate );
  }
          
    
    
  
    public User getUser() {
    return tutorial.getUser();
  }
  public void setUser( User user ) {
    tutorial.setUser( user );
  }
        
/* tutorial - generated by stajanov (do not edit/delete) */
}
